package com.usky.oa.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.OaApproval;
import com.usky.oa.domain.OaDocument;
import com.usky.oa.domain.OaFormDefinition;
import com.usky.oa.mapper.OaApprovalMapper;
import com.usky.oa.mapper.OaDocumentMapper;
import com.usky.oa.mapper.OaFormDefinitionMapper;
import com.usky.oa.service.OaApprovalService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.oa.service.utils.OaProcessAnalysis;
import com.usky.oa.service.utils.OaSendMessageCenter;
import com.usky.oa.service.vo.OaApprovalCountVO;
import com.usky.system.domain.SysUser;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.validator.internal.constraintvalidators.bv.time.futureorpresent.FutureOrPresentValidatorForReadableInstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 审批表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-12-10
 */
@Service
public class OaApprovalServiceImpl extends AbstractCrudService<OaApprovalMapper, OaApproval> implements OaApprovalService {

    @Autowired
    private OaApprovalMapper oaApprovalMapper;

    @Autowired
    private OaDocumentMapper oaDocumentMapper;

    @Autowired
    private OaSendMessageCenter oaSendMessageCenter;

    @Autowired
    private OaProcessAnalysis oaProcessAnalysis;

    @Autowired
    private OaFormDefinitionMapper oaFormDefinitionMapper;


    // 我的申请-待处理、已处理、我发起、我收到-统计(暂时统计近一年数据)
    @Override
    public OaApprovalCountVO approvalCount() {
        Long userId = SecurityUtils.getUserId();
        String username = SecurityUtils.getUsername();
        Integer tenantId = SecurityUtils.getTenantId();

        LocalDateTime now = LocalDateTime.now();
        // 一年前时间
        LocalDateTime oneYearAgo = now.minusYears(1);

        OaApprovalCountVO oaApprovalCountVO = new OaApprovalCountVO();
        // 待处理
        oaApprovalCountVO.setPendingSum(getPendingSum(tenantId, userId, oneYearAgo, now).size());
        // 已处理
        oaApprovalCountVO.setAlreadySum(getAlreadySum(tenantId, userId, oneYearAgo, now).size());
        // 我发起
        oaApprovalCountVO.setMyInitiated(getMyInitiated(tenantId, username, oneYearAgo, now).size());
        // 我收到
        oaApprovalCountVO.setMyReceived(getMyReceived(tenantId, userId, oneYearAgo, now).size());

        return oaApprovalCountVO;
    }

    // 我的代办-待处理、已处理、我收到-分页
    @Override
    public CommonPage<OaApproval> pageList(Integer queryType, String docNo, String formSign, Integer approvalStatus, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();
        IPage<OaApproval> page = new Page<>(pageNum, pageSize);

        List<Integer> processed = Stream.of(2, 3).collect(Collectors.toList());

        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getTenantId, tenantId)
                .eq(OaApproval::getApprovalUid, userId).orderByDesc(OaApproval::getApprovalDate);

        switch (queryType) {
            case 1:
                queryWrapper.eq(OaApproval::getApprovalStatus, 1)
                        .like(StringUtils.isNotBlank(docNo), OaApproval::getDocNo, docNo)
                        .like(StringUtils.isNotBlank(formSign), OaApproval::getDocNo, formSign)
                        .between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), OaApproval::getSubmitDate, startTime, endTime);
                break;
            case 2:
                queryWrapper.like(StringUtils.isNotBlank(docNo), OaApproval::getDocNo, docNo)
                        .like(StringUtils.isNotBlank(formSign), OaApproval::getDocNo, formSign)
                        .eq(approvalStatus != null && approvalStatus > 1 && approvalStatus < 4, OaApproval::getApprovalStatus, approvalStatus)
                        .in(OaApproval::getApprovalStatus, processed)
                        .between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), OaApproval::getSubmitDate, startTime, endTime);
                break;
            case 3:
                queryWrapper.like(StringUtils.isNotBlank(docNo), OaApproval::getDocNo, docNo)
                        .like(StringUtils.isNotBlank(formSign), OaApproval::getDocNo, formSign)
                        .eq(OaApproval::getType, 1)
                        .eq(approvalStatus != null && approvalStatus > 0 && approvalStatus < 4, OaApproval::getApprovalStatus, approvalStatus)
                        .between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), OaApproval::getSubmitDate, startTime, endTime);
                break;
            default:
                throw new RuntimeException("查询类型参数错误");
        }
        return ToCommonPage(oaApprovalMapper.selectPage(page, queryWrapper));
    }

    // 审批
    @Override
    public void approve(Integer id, Integer approvalStatus, String opinion) {
        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getId, id);
        OaApproval oaApproval = oaApprovalMapper.selectOne(queryWrapper);

        if (!oaApproval.getApprovalUid().equals(SecurityUtils.getUserId())) {
            throw new BusinessException("审批权限不足，暂时无法更改！");
        }

        if (oaApproval.getApprovalStatus() != 1) {
            throw new BusinessException("审批状态异常，暂时无法更改！");
        }
        if (approvalStatus < 2 || approvalStatus > 3) {
            throw new BusinessException("审批类型错误，请重试！");
        }

        String username = SecurityUtils.getUsername();

        LocalDateTime now = LocalDateTime.now();

        oaApproval.setApprovalStatus(approvalStatus);
        oaApproval.setOpinion(opinion);
        oaApproval.setApprovalDate(now);
        oaApproval.setUpdateBy(username);
        oaApproval.setUpdateTime(now);
        oaApprovalMapper.updateById(oaApproval);

        int isPass = 1;
        if (approvalStatus == 3) {
            isPass = 0;
        }

        List<Long> receivers = Collections.singletonList(Long.valueOf(oaApproval.getProposer()));
        oaSendMessageCenter.sendAsyncMessage(oaApproval.getCreateBy(), Long.valueOf(oaApproval.getProposer()), oaApproval.getId(), receivers, isPass);
    }

    @Override
    public void analysisTest(Integer id) {
        LambdaQueryWrapper<OaFormDefinition> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaFormDefinition::getId, id);
        String flowInfo = oaFormDefinitionMapper.selectOne(queryWrapper).getFlowInfo();
        oaProcessAnalysis.processAnalysis(flowInfo);
    }

    // 待处理
    private List<OaApproval> getPendingSum(Integer tenantId, Long userId, LocalDateTime oneYearAgo, LocalDateTime now) {
        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getTenantId, tenantId)
                .eq(OaApproval::getApprovalUid, userId)
                .eq(OaApproval::getApprovalStatus, 1)
                .between(OaApproval::getSubmitDate, oneYearAgo, now);
        List<OaApproval> oaApprovals = oaApprovalMapper.selectList(queryWrapper);
        return oaApprovals == null ? Collections.emptyList() : oaApprovals;
    }

    // 已处理
    private List<OaApproval> getAlreadySum(Integer tenantId, Long userId, LocalDateTime oneYearAgo, LocalDateTime now) {
        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getTenantId, tenantId)
                .eq(OaApproval::getApprovalUid, userId)
                .and(wrapper -> wrapper
                        .eq(OaApproval::getApprovalStatus, 2)
                        .or()
                        .eq(OaApproval::getApprovalStatus, 3))
                .between(OaApproval::getSubmitDate, oneYearAgo, now);
        List<OaApproval> oaApprovals = oaApprovalMapper.selectList(queryWrapper);
        return oaApprovals == null ? Collections.emptyList() : oaApprovals;
    }

    // 我发起
    /*private List<OaApproval> getMyInitiated(Integer tenantId, String username) {
        List<String> docNoList = oaDocumentMapper.selectList(new QueryWrapper<OaDocument>().lambda()
                .select(OaDocument::getDocNo)
                .eq(OaDocument::getTenantId, tenantId)
                .eq(OaDocument::getCreateBy, username)
                // 已发起只计算 审批中、审通过、审批不通过
                .in(OaDocument::getDocStatus, 1, 2, 3)).stream().map(OaDocument::getDocNo).collect(Collectors.toList());

        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(OaApproval::getDocNo, docNoList);
        List<OaApproval> oaApprovals = oaApprovalMapper.selectList(queryWrapper);
        return oaApprovals == null ? Collections.emptyList() : oaApprovals;
    }*/

    // 我发起
    private List<OaDocument> getMyInitiated(Integer tenantId, String username, LocalDateTime oneYearAgo, LocalDateTime now) {
        List<OaDocument> oaDocuments = oaDocumentMapper.selectList(new QueryWrapper<OaDocument>().lambda()
                .select(OaDocument::getDocNo)
                .eq(OaDocument::getTenantId, tenantId)
                .eq(OaDocument::getCreateBy, username)
                .in(OaDocument::getDocStatus, 1, 2, 3)
                .between(OaDocument::getCreateTime, oneYearAgo, now));
        return oaDocuments == null ? Collections.emptyList() : oaDocuments;
    }

    // 我收到
    private List<OaApproval> getMyReceived(Integer tenantId, Long userId, LocalDateTime oneYearAgo, LocalDateTime now) {
        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getTenantId, tenantId)
                .eq(OaApproval::getApprovalUid, userId)
                .eq(OaApproval::getType, 1)
                .between(OaApproval::getSubmitDate, oneYearAgo, now);
        List<OaApproval> oaApprovals = oaApprovalMapper.selectList(queryWrapper);
        return oaApprovals == null ? Collections.emptyList() : oaApprovals;
    }

}
